/*
Dozen Days of Words episode 3: Hangman
Copyright (c) 2009 Blazing Games Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.dozendays.words.hangman
{
	import flash.display.Sprite;

	public class HangmanDrawer
	extends Sprite
	{
// ***** CONSTANTS **********************************************************

		public static const SECTION_START:int = 0;
		public static const SECTION_GROUND:int = 1;
		public static const SECTION_BASE:int = 2;
		public static const SECTION_BRANCH:int = 3;
		public static const SECTION_SUPPORT:int = 4;
		public static const SECTION_ROPE:int = 5;
		public static const SECTION_HEAD:int = 6;
		public static const SECTION_BODY:int = 7;
		public static const SECTION_LEFTARM:int = 8;
		public static const SECTION_RIGHTARM:int = 9;
		public static const SECTION_LEFTLEG:int = 10;
		public static const SECTION_RIGHTLEG:int = 11;

		public static const EASY_STEPS:Array = new Array(
				0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11 );
		public static const NORMAL_STEPS:Array = new Array(
				0, 2, 4, 6, 7, 8, 9, 10, 11 );
		public static const HARD_STEPS:Array = new Array(
				0, 6, 7, 8, 9, 10, 11 );
		

// ***** VARIABLES **********************************************************

		protected var _sectionState:int;
		protected var _size:int;
		protected var _thickness:int;
		protected var _colour:uint;

// ***** CONSTRUCTOR ********************************************************

		public function HangmanDrawer(size:int = 140, thickness:int = 10, startState:int = 0, colour:uint = 0x0000FF)
		{
			_sectionState = startState;
			_size = size;
			_thickness = thickness;
			_colour = colour;
			draw();
		}

// ***** Functions - Puzzle Handling ****************************************

		public function setHangingState(sectionState:int):void {
			_sectionState = sectionState;
			draw();
		}

		private function draw():void{
			var multiplier:Number = _size / 14;
			
			graphics.clear();
			graphics.lineStyle(_thickness, _colour);
			switch (_sectionState) {
			case SECTION_RIGHTLEG:
				graphics.moveTo(9*multiplier, 8*multiplier);
				graphics.lineTo(12*multiplier, 13*multiplier);

			case SECTION_LEFTLEG:
				graphics.moveTo(9*multiplier, 8*multiplier);
				graphics.lineTo(6*multiplier, 13*multiplier);

			case SECTION_RIGHTARM:
				graphics.moveTo(9*multiplier, 4*multiplier);
				graphics.lineTo(12*multiplier, 6*multiplier);

			case SECTION_LEFTARM:
				graphics.moveTo(9*multiplier, 4*multiplier);
				graphics.lineTo(6*multiplier, 6*multiplier);

			case SECTION_BODY:
				graphics.moveTo(9*multiplier, 4*multiplier);
				graphics.lineTo(9*multiplier, 8*multiplier);

			case SECTION_HEAD:
				graphics.drawEllipse(8*multiplier, 2*multiplier,2*multiplier, 2*multiplier);

			case SECTION_ROPE:
				graphics.moveTo(9*multiplier, 0*multiplier);
				graphics.lineTo(9*multiplier, 2*multiplier);

			case SECTION_SUPPORT:
				graphics.moveTo(1*multiplier, 2*multiplier);
				graphics.lineTo(3*multiplier, 0*multiplier);

			case SECTION_BRANCH:
				graphics.moveTo(1*multiplier, 0*multiplier);
				graphics.lineTo(9*multiplier, 0*multiplier);

			case SECTION_BASE:
				graphics.moveTo(1*multiplier, 0*multiplier);
				graphics.lineTo(1*multiplier, 14*multiplier);

			case SECTION_GROUND:
				graphics.moveTo(0*multiplier, 14*multiplier);
				graphics.lineTo(3*multiplier, 14*multiplier);

			}
		}
	}
}
