/*
Dozen Days of Words episode 3: Hangman
Copyright (c) 2009 Blazing Games Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.dozendays.words.hangman
{
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;
	import flash.utils.getTimer;

	import com.blazinggames.dozendays.TextButton;
	import com.blazinggames.dozendays.YesNoDialog;
	import com.blazinggames.dozendays.words.LetterTile;

	public class HangmanGame
	extends Sprite
	{
// ***** CONSTANTS **********************************************************

		public static const DONE_TEXT:String = "Return to Title!";
		public static const WON_TEXT:String = "You Have Won!";
		public static const LOST_TEXT:String = "You Have Lost!";

		public static const LETTERS:String = "ABCDEFGHIJKLMNOPQRSTUVWXYZ*";
		public static const TILE_SIZE:int = 35;
		public static const HORIZONAL_GAP:int = 5;
		public static const VERTICAL_GAP:int = 10;
		public static const NUM_ROWS:int = 3;
		public static const LETTERS_PER_ROW:int = 9;
		public static const FIRST_TILE_X:int = 500;
		public static const FIRST_TILE_Y:int = 50;
		public static const HANGMAN_X:int = 75;
		public static const HANGMAN_Y:int = 20;
		
		public static const PUZZLE_SIZE:int = 32;
		public static const MAX_PUZZLE_LETTERS:int = 20;
		public static const PUZZLE_X:int = 0;
		public static const PUZZLE_Y:int = 440;
		
		public static const BACKGROUND_COLOR:uint = 0x000050;
		public static const TILE_COLOR:uint = 0x500050;
		public static const LETTER_COLOR:uint = 0xC000C0;
		public static const TILE_COLOR_DIM:uint = 0x200020;
		public static const LETTER_COLOR_DIM:uint = 0x500050;
		public static const HIGHLIGHT_COLOR:uint = 0xFF0050;
		public static const WORD_COLOR:uint = 0x00C000;

		public static const PUZZLES:Array = new Array(
			"HANGMAN", "BLAZING GAMES", "APRIL FOOL", "CANADA DAY GAME", 
			"DARTS", "DESKTOP TENNIS", "EVIL EGGS", "HALLOWEEN LICH",
			"INDEPENDENCE DAY", "LEAP YEAR LEAP", "LIGHTS OUT", 
			"MISSILE MASTER", "MR. HOLEMAN", "SANTA\'S SNOWBALL", 
			"STRING ALONG", "THIRTEEN SPIKES", "VALENTINE HEART", 
			"VAMPIRE ATTACK", "BOMB NIM", "CHRISTMAS TWIST", "COLOR COLLAPSE",
			"DEADLY DOORS", "KNIGHTS TOUR", "MEMORY CHALLENGE", "NIM", 
			"PICTURE PUZZLE", "QUEEN\'S CHALLENGE", "ROTATION PUZZLE",
			"SLIDING PUZZLE", "TOWER OF HANOI", "WILMAG\'S LAB", 
			"3D TIC TAC TOE", "BIG WHEEL", "CRIBBAGE SQUARE", "FLEET",
			"FOUR IN A ROW", "PENT UP ANGER", "POKER SQUARE", "SOLITAIRE",
			"VIDEO POKER", "COFFEE QUEST", "SANTA'S SEARCH", "TENNIS WITH DICE",
			"THIRTY SIX", "THREES", "POKER DICE", "SIC BO", "DROP DEAD", 
			"HEARTS", "SHUT THE BOX", "PIG", "GENERAL", "EVEREST", "DIX MILLE",
			"SCRAMBLED PYRAMID", "ALPHABET POINTS", "ONE OF THOSE WEEKS",
			"BIORHYTHMS", "EGG GENERATOR", "LUCKY SCRATCH", "FORTUNE COOKIE",
			"HALLOWEEN SCRATCH", "DOORS", "HIDE AND SEEK", "NUCLEAR TERRORISM",
			"ARTILLERY BATTLE", "TANKS", "TANKS FOR MEMORY", "CHUCK A LUCK",
			"BLACKJACK", "ROULETTE", "CRAPS", "BACCARAT", "SLOTS", 
			"FIVE D EIGHT", "CHECKERS", "PYRAMID SOLITAIRE", "BACKGAMMON",
			"REVERSI", "CRIBBAGE", "WORD SCRAMBLE", "DOTS", "CYPHER", 
			"LIGHT BOX", "CIRCLE WORD", "MINEFIELD", "MAZE", "MONSTER HUNT",
			"HOTDOG STAND", "MELEE COMBAT", "ADVENTURE", "DUNGEON ROMP"

		);
		
// ***** VARIABLES **********************************************************

		protected var _hangman:Hangman;
		protected var _doneButton:TextButton;
		protected var _tiles:Array;
		protected var _backdrop:TextField;

		protected var _currentRound:int;
		protected var _word:Array;

		protected var _drawOrder:Array;
		protected var _drawingIndex:int;
		protected var _hangmanDrawer:HangmanDrawer;

		protected var _haveWon:TextField;

		protected var _currentSolution:String;
		protected var _currentPuzzle:String;
		protected var _chosenLetters:String;
		
// ***** CONSTRUCTOR ********************************************************

		public function HangmanGame(hangman:Hangman)
		{
			var cntrRow:int, cntrCol:int, rowLength:int, tileX:int, tileY:int;
			_hangman = hangman;

			_backdrop = new TextField();
			_backdrop.multiline = true;
			_backdrop.width = 640;
			_backdrop.height = 480;
			_backdrop.textColor = 0xC000C0;
			_backdrop.background = true;
			_backdrop.backgroundColor = BACKGROUND_COLOR;
			addChild(_backdrop);
			
			_tiles = new Array();
			var indx:int = 0;
			var wordFormat:TextFormat = new TextFormat(null, 20, WORD_COLOR, null, null, null, null, null, TextFormatAlign.CENTER);
			for (cntrRow = 0; cntrRow < NUM_ROWS; ++cntrRow) {
				tileX = FIRST_TILE_X + (cntrRow * (TILE_SIZE + VERTICAL_GAP));
				tileY = FIRST_TILE_Y;
				for (cntrCol = 0; cntrCol < LETTERS_PER_ROW; ++cntrCol) {
					_tiles[indx] = new LetterTile(TILE_SIZE,TILE_SIZE, LETTERS.substring(indx, indx+1));
					_tiles[indx].x = tileX;
					_tiles[indx].y = tileY;
					_tiles[indx].setColors(LETTER_COLOR, TILE_COLOR);
					_tiles[indx].addEventListener(LetterTile.TILE_SELECTED_EVENT, handleTileSelection);
					addChild(_tiles[indx]);

					tileY += TILE_SIZE;
					tileY += HORIZONAL_GAP;
					++indx;
				}
				
			}
			removeChild(_tiles[26]);

			_word = new Array();
			for (cntrCol = 0; cntrCol < MAX_PUZZLE_LETTERS; ++cntrCol) { 
				_word[cntrCol] = new LetterTile(PUZZLE_SIZE,PUZZLE_SIZE, "*");
				_word[cntrCol].x = PUZZLE_X + PUZZLE_SIZE * cntrCol;
				_word[cntrCol].y = PUZZLE_Y;
				_word[cntrCol].setColors(WORD_COLOR, BACKGROUND_COLOR);
				addChild(_word[cntrCol]);
			}
			
			var hangmanBackdrop:HangmanDrawer = new HangmanDrawer(400,2,11,0x000080);
			hangmanBackdrop.x = HANGMAN_X;
			hangmanBackdrop.y = HANGMAN_Y;
			addChild(hangmanBackdrop);
			_hangmanDrawer = new HangmanDrawer(400,20,11);
			_hangmanDrawer.x = HANGMAN_X;
			_hangmanDrawer.y = HANGMAN_Y;
			addChild(_hangmanDrawer);
			_doneButton = new TextButton(DONE_TEXT, 200, 40);
			_doneButton.x = 440;
			_doneButton.y = 0;
			addChild(_doneButton);
			_doneButton.addEventListener(MouseEvent.MOUSE_UP, handleExit);

			var winFormat:TextFormat = new TextFormat(null, 90, 0xFFFF00, null, null, null, null, null, TextFormatAlign.CENTER);
			_haveWon = new TextField();
			_haveWon.width = 640;
			_haveWon.height = 150;
			_haveWon.y = 200;
			_haveWon.defaultTextFormat = winFormat;
			_haveWon.text = WON_TEXT;
			_haveWon.visible = false;
			addChild(_haveWon);
		}

// ***** Functions - Puzzle Handling ****************************************

		public function startGame(playmode:int = 3):void
		{
			var cntr:int;
			var puzzleID:int = Math.floor(Math.random() * PUZZLES.length);
			var letter:String;

			_currentSolution = PUZZLES[puzzleID];
			_currentPuzzle = "";
			for (cntr = 0; cntr < _currentSolution.length; ++cntr) {
				letter = _currentSolution.substr(cntr, 1);
				
				if (LETTERS.indexOf(letter) >= 0)
					letter = "_";
				_currentPuzzle += letter;
				_word[cntr].setLetter(letter);
				_word[cntr].setColors(WORD_COLOR, BACKGROUND_COLOR);
			}
			for (cntr = _currentSolution.length; cntr < MAX_PUZZLE_LETTERS; ++cntr)
				_word[cntr].setLetter(" ");
//			_backdrop.text = (_currentPuzzle + " -> " + _currentSolution);
			_chosenLetters = "";
			for (cntr = 0; cntr < _tiles.length; ++cntr)
				_tiles[cntr].setColors(LETTER_COLOR, TILE_COLOR);

			if (playmode == HangmanTitle.PLAY_EASY)
				_drawOrder = HangmanDrawer.EASY_STEPS;
			else if (playmode == HangmanTitle.PLAY_HARD)
				_drawOrder = HangmanDrawer.HARD_STEPS;
			else
				_drawOrder = HangmanDrawer.NORMAL_STEPS;
			
			_drawingIndex = 0;
			_hangmanDrawer.setHangingState(_drawOrder[0]);
			_haveWon.visible = false;
		}


		private function winloseGame(s:String, c:uint = 0xFF0000):void {
			_chosenLetters = LETTERS;
			for (var cntr:int = 0; cntr < _tiles.length; ++cntr)
				_tiles[cntr].setColors(LETTER_COLOR_DIM, TILE_COLOR_DIM);
			_haveWon.text = s;
			_haveWon.visible = true;
			
			var letter:String;
			for (cntr = 0; cntr < _currentSolution.length; ++cntr) {
				letter = _currentSolution.substr(cntr, 1);
				_word[cntr].setLetter(letter);
				_word[cntr].setColors(c, BACKGROUND_COLOR);
			}
		}
		
// ***** Functions - Event Handling *****************************************

		private function handleExit(evt:Event):void
		{
			_hangman.changeMode(Hangman.MODE_TITLE);
		}
		

		private function handleTileSelection(evt:Event):void
		{
			var cntr:int;
			var letter:String;
			var selected:LetterTile = LetterTile(evt.target);
			selected.setColors(LETTER_COLOR_DIM, TILE_COLOR_DIM);
			letter = selected.getLetter();
			
			if (_chosenLetters.indexOf(letter) >= 0)
				return;
			else {
				_chosenLetters += letter;
			}
			var isAMatch:Boolean = false;
			var missingLetterCount:int = 0;
			for (cntr = 0; cntr < _currentSolution.length; ++cntr) {
				if (letter == _currentSolution.substr(cntr, 1)) {
					_word[cntr].setLetter(letter);
					isAMatch = true;
				}
				if (_word[cntr].getLetter() == "_")
					++missingLetterCount;
			}
			if (missingLetterCount == 0)
				winloseGame(WON_TEXT, 0xFFFF88);
			if (!isAMatch) {
				++_drawingIndex;
				_hangmanDrawer.setHangingState(_drawOrder[_drawingIndex]);
				if (_drawingIndex == (_drawOrder.length - 1))
					winloseGame(LOST_TEXT);
			}

		}
		
	}
}
